/***********************/
/* nimutils.c          */
/* for LoseNIM 1.00    */
/* by Adam M. Costello */
/***********************/

/* This is ANSI C code. */


#include "nimutils.h"  /* Makes sure we're consistent with prototypes, */
                       /* and also includes "errmsg.h".                */

#include <stdlib.h>
#include <string.h>


struct nimstate {
  nimpile piles, lastpile;
  nim_uint xor, total, numplural;
};

struct nimpile {
  nimpile next;
  nimstate state;
  nim_uint size;
};

const nimstate nostate = (nimstate) 0;
const nimpile  nopile  = (nimpile) 0;


nimstate newstate(errmsg_t errmsg)
{
  nimstate state = nostate;

  state = malloc(sizeof (struct nimstate));

  if (state) {
    state->piles = state->lastpile = nopile;
    state->xor = state->total = 0;
    state->numplural = 0;
    *errmsg = '\0';
  }
  else strcpy(errmsg,outofmem);

  return state;
}


void freestate(nimstate state)
{
  nimpile pile, nextpile;

  for (pile = state->piles;  pile;  pile = nextpile) {
    nextpile = pile->next;
    free(pile);
  }

  free(state);
}


nimpile newpile(nimstate state, nim_uint numstones, errmsg_t errmsg)
{
  nimpile pile;

  pile = malloc(sizeof (struct nimpile));

  if (pile) {
    if (state->lastpile)
      state->lastpile = state->lastpile->next = pile;
    else
      state->lastpile = state->piles = pile;
    pile->next = nopile;
    pile->state = state;
    pile->size = numstones;
    state->total += numstones;
    state->xor ^= numstones;
    if (numstones > 1) ++state->numplural;
    *errmsg = '\0';
  }
  else strcpy(errmsg,outofmem);

  return pile;
}


nimpile firstpile(nimstate state)
{
  return state->piles;
}


nimpile nextpile(nimpile pile)
{
  return pile->next;
}


nimpile findpile(nimstate state, nim_uint numstones)
{
  nimpile pile;

  for (pile = state->piles;  pile;  pile = pile->next)
    if (pile->size == numstones) break;

  return pile;
}


nim_uint pilesize(nimpile pile)
{
  return pile->size;
}


nim_uint diminishpile(nimpile pile, nim_uint numstones)
{
  nimstate state = pile->state;

  if (pile->size >= numstones) {
    state->xor ^= pile->size;
    if (pile->size > 1) --state->numplural;
    pile->size -= numstones;
    state->total -= numstones;
    state->xor ^= pile->size;
    if (pile->size > 1) ++state->numplural;
  }

  return pile->size;
}


nim_uint totalstones(nimstate state)
{
  return state->total;
}


nim_uint nimxor(nimstate state)
{
  return state->xor;
}


nim_uint numpluralpiles(nimstate state)
{
  return state->numplural;
}
