/***********************/
/* nimutils.h          */
/* for LoseNIM 1.00    */
/* by Adam M. Costello */
/***********************/

/* This is ANSI C code. */


/* In all of the functions declared here, it is the caller's    */
/* responsibility to pass valid nimstates and nimpiles, which   */
/* can only be obtained as return values from the functions.    */
/* The constants nostate and nopile are *not* considered valid. */


#include "errmsg.h"


/* nim_uint is guaranteed to be an unsigned integral type. */

typedef unsigned int nim_uint;
#define NIM_UINT_SCANF_SPEC "%u"

typedef struct nimstate *nimstate;
typedef struct nimpile *nimpile;

extern const nimstate nostate;
extern const nimpile  nopile;


nimstate newstate(errmsg_t errmsg);

/* newstate() returns an identifier for the state of a NIM      */
/* game, or nostate on failure.  The state consists of 0 piles. */


void freestate(nimstate state);

/* freestate(state) frees resources associated with state.        */
/* Neither state nor its piles may be referenced after this call. */


nimpile newpile(nimstate state, nim_uint numstones, errmsg_t errmsg);

/* newpile(state,numstones) adds a new pile of n stones to     */
/* state, returning an identifier for the new pile on success. */
/* On failure, nopile is returned and state is unaffected.     */


nimpile firstpile(nimstate state);

/* firstpile(state) returns an identifier for the first pile */
/* of stones belonging to state, or nopile if there is none. */


nimpile nextpile(nimpile pile);

/* nextpile(pile) returns an identifier for the next */
/* pile after pile, or nopile if there is none.      */


nimpile findpile(nimstate state, nim_uint numstones);

/* findpile(state,numstones) returns an identifier for a pile */
/* belonging to state containing exactly numstones stones if  */
/* there is such a pile, or nopile if there is not.           */


nim_uint pilesize(nimpile pile);

/* pilesize(pile) returns the number of stones in pile. */


nim_uint diminishpile(nimpile pile, nim_uint numstones);

/* diminishpile(pile,numstones) reduces the number   */
/* of stones in pile by numstones if pile currently  */
/* contains at least that many stones, otherwise it  */
/* leaves all the stones there.  In either case, the */
/* number of stones left in the pile is returned.    */


nim_uint totalstones(nimstate state);

/* totalstones(state) returns the total    */
/* number of stones in all piles in state. */


nim_uint nimxor(nimstate state);

/* nimxor(state) returns the bitwise exclusive OR of the numbers */
/* of stones in each of its piles, or 0 if there are no piles.   */


nim_uint numpluralpiles(nimstate state);

/* numpluralpiles(state) returns the number of piles  */
/* belonging to state containing more than one stone. */
